unit ConOpts;
(*##*)
(*******************************************************************
*                                                                 *
*   C  O  N  O  P  T  S   Comport option set dialog                *
*                         Part of EPager                          *
*                                                                  *
*                                                                 *
*   Copyright (c) 1998, A.Ivanov. All rights reserved.             *
*   Delphi 2, 3 only (uses huge strings)                          *
*   Conditional defines: ASYNC H+                                  *
*   define MSCOMM if wants MSComm component                       *
*   Last Revision: Mar 19 1998                                     *
*   Last fix     : Mar 19 1998                                    *
*   Lines        : 401                                             *
*   History      : see CHANGES.TXT file                           *
*   Printed      : ---                                             *
*                                                                 *
********************************************************************)
(*##*)

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Mask, ComCtrls,
  tap;

const
  iniPortNo          = 'Port';
  iniPortSetS        = 'PortSettings';
  iniCDWaitSec       = 'CarrierWaitSeconds';
  iniHandShakingType = 'Handshake';
  iniPhone           = 'ModemDialStr';
  iniLoginPrompt     = 'Prompt';
  iniPasswd          = 'Passwd';{ ?!!}
  iniUserName        = 'UserName';
  iniManualLogStr    = 'Switch2Manual';
  iniReadyTransaction= 'ReadyTransactionStr';
  iniOptions         = 'TerminalSpecific';

type
  TFormConnectOptions = class(TForm)
    EPars: TEdit;
    EPhone: TEdit;
    EPasswd: TEdit;
    ELoginPrompt: TEdit;
    EReadyTransaction: TEdit;
    EID: TMaskEdit;
    EMManualLogon: TEdit;
    CBASCIITextOnly: TCheckBox;
    CBNoWaitReady: TCheckBox;
    CBNoSendMsg: TCheckBox;
    CBNoWaitAccept: TCheckBox;
    CBPhone: TCheckBox;
    CBWaitPrompt: TCheckBox;
    CBManualLogon: TCheckBox;
    PCAutoOrManualLogon: TPageControl;
    TSAutoLogon: TTabSheet;
    TSManualLogon: TTabSheet;
    TSTransmit: TTabSheet;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Image6: TImage;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    LAutoLogonEnabled: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Panel1: TPanel;
    OKBtn: TButton;
    CancelBtn: TButton;
    PageControl2: TPageControl;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet8: TTabSheet;
    Button1: TButton;
    GroupBox1: TGroupBox;
    CBFlowNone: TCheckBox;
    CBFlowXOnXOff: TCheckBox;
    CBFlowRTSCTS: TCheckBox;
    CBFlowDTRDSR: TCheckBox;
    GBFlowChar: TGroupBox;
    EXOn: TEdit;
    UpDown1: TUpDown;
    EXOff: TEdit;
    UpDown2: TUpDown;
    Label2: TLabel;
    UpDown3: TUpDown;
    UpDown4: TUpDown;
    EPort: TEdit;
    ECDWaitSec: TEdit;
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CBPhoneClick(Sender: TObject);
    procedure CBNoWaitReadyClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure CBManualLogonClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure CBFlowClick(Sender: TObject);
    procedure CBFlowNoneClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    ConnectPars: TConnectPars;
    procedure FillFields(AReturn: Boolean);
    procedure StoreIni;
    procedure LoadIni;
  end;

var
  FormConnectOptions: TFormConnectOptions;

function FormConOptionsDlg(var AConnectPars: TConnectPars): Boolean;

{  ASection = ''    iniTAPSECTION ([TAP])
         
   
}
function ConLoadIni(const ASection: String; var AConnectPars: TConnectPars): Boolean;
function ConStoreIni(const ASection: String; var AConnectPars: TConnectPars): Boolean;

implementation

uses
  IniFiles,
  util1, PagOpts;

{$R *.DFM}

procedure TFormConnectOptions. FillFields(AReturn: Boolean);
begin
  with ConnectPars do if AReturn then begin
    {       }
    PortNo1 := StrToInt(EPort.Text);
    PortSetS:= EPars.Text;
    CDWaitSec:= StrToInt(ECDWaitSec.Text);
    HandShakingType:= 0;
    if not CBFlowNone.Checked then begin
      if CBFlowXOnXOff.Checked then HandShakingType:= 1;
      if CBFlowRTSCTS.Checked then Inc(HandShakingType, 2);
      if CBFlowDTRDSR.Checked then Inc(HandShakingType, 4);
    end;
    XONChar:= StrToIntDef(EXOn.Text, 17);
    XOFFChar:= StrToIntDef(EXOff.Text, 19);

    Phone:= EPhone.Text;
    LoginPrompt:= ELoginPrompt.Text;
    UserName:= EID.Text;
    Passwd:= EPasswd.Text;
    PgConOptSet:= [];
    if CBASCIITextOnly.Checked
    then Include(PgConOptSet, cmASCIITextOnly);
    if CBPhone.Checked
    then Include(PgConOptSet, cmThruModem);
    if CBWaitPrompt.Checked
    then Include(PgConOptSet, cmWaitPrompt);
    if CBManualLogon.Checked
    then Include(PgConOptSet, cmManualLogin);
    if CBNoWaitReady.Checked
    then Include(PgConOptSet, cmNoWaitReady);
    if CBNoWaitAccept.Checked
    then Include(PgConOptSet, cmNoWaitAccept);
    if CBNoSendMsg.Checked
    then Include(PgConOptSet, cmNoSendMsg);
    {
    if CB8BitCRC.Checked
    then Include(PgConOptSet, cm8BitCRC);
    }
    ManualLogStr:= EMManualLogon.Text;
    ReadyTransactionStr:= EReadyTransaction.Text;
  end else begin
    {     }
    {  PortNo = 0      ! }
    if PortNo1 > 0 then begin
      EPort.Text:= IntToStr(PortNo1);
      EPars.Text:= PortSetS;
      ECDWaitSec.Text:= IntToStr(CDWaitSec);
      CBFlowNone.Checked:= HandShakingType = 0;
      CBFlowXOnXOff.Checked:= HandShakingType and 1 = 1;
      CBFlowRTSCTS.Checked:= HandShakingType and 2 = 2;
      CBFlowDTRDSR.Checked:= HandShakingType and 4 = 4;
      EXOn.Text:= IntToStr(XOnChar);
      EXOff.Text:= IntToStr(XOFFChar);
      EPhone.Text:= Phone;
      ELoginPrompt.Text:= LoginPrompt;
      EID.Text:= UserName;
      EPasswd.Text:= Passwd;
      CBASCIITextOnly.Checked:= cmASCIITextOnly in PgConOptSet;
      CBPhone.Checked:= cmThruModem in PgConOptSet;
      CBWaitPrompt.Checked:= cmWaitPrompt in PgConOptSet;
      CBManualLogon.Checked:= cmManualLogin in PgConOptSet;
      CBNoWaitReady.Checked:= cmNoWaitReady in PgConOptSet;
      CBNoWaitAccept.Checked:= cmNoWaitAccept in PgConOptSet;
      CBNoSendMsg.Checked:= cmNoSendMsg in PgConOptSet;
      {
      CB8BitCRC.Checked:= cm8BitCRC in PgConOptSet;
      }
      EMManualLogon.Text:= ManualLogStr;
      EReadyTransaction.Text:= ReadyTransactionStr;
    end;
  end;
end;

{ fill up:
  if return = true then fill up (return):
}
function FormConOptionsDlg(var AConnectPars: TConnectPars): Boolean;
begin
  Result:= False;
  FormConnectOptions:= TFormConnectOptions.Create(Application);
  with FormConnectOptions do begin
    {   }
    ConnectPars:= AConnectPars;
    { FormActivate  controls   New.. }
    if ShowModal = mrOK then begin
      {          }
      AConnectPars:= ConnectPars;
      Result:= True;
    end;
  end;
  FormPagerOptions.Free;
end;

function ConLoadIni(const ASection: String; var AConnectPars: TConnectPars): Boolean;
var
  Ini: TIniFile;
  L: LongInt;
  SectName: String;
begin
  Ini:= TIniFile.Create(iniPagerOPTINIFN);
  if ASection = ''
  then SectName:= iniTAPSECTION
  else SectName:= ASection;
  with Ini, AConnectPars do begin
    {          }
    PortNo1:= ReadInteger(SectName, iniPortNo, 0);
    PortSetS:= ReadString(SectName, iniPortSetS, '');
    CDWaitSec:= ReadInteger(SectName, iniCDWaitSec, 0);
    HandShakingType:= ReadInteger(SectName, iniHandShakingType, 0);
    Phone:= ReadString(SectName, iniPhone, '');
    LoginPrompt:= ReadString(SectName, iniLoginPrompt, '');
    Passwd:= ReadString(SectName, iniPasswd, '');
    UserName:= ReadString(SectName, iniUserName, '');
    ManualLogStr:= ReadString(SectName, iniManualLogStr, '');
    ReadyTransactionStr:= ReadString(SectName, iniReadyTransaction, '');
    L:= ReadInteger(SectName, iniOptions, 0);
    PgConOptSet:=  TPgConOptSet((@L)^);
  end;
  Ini.Free;
  Result:= True;
end;

function ConStoreIni(const ASection: String; var AConnectPars: TConnectPars): Boolean;
var
  Ini: TIniFile;
  SectName: String;
begin
  Result:= False;
  Ini:= TIniFile.Create(iniPagerOPTINIFN);
  if ASection = ''
  then SectName:= iniTAPSECTION
  else SectName:= ASection;
  with Ini, AConnectPars do begin
    {          }
    WriteInteger(SectName, iniPortNo, PortNo1);
    WriteString(SectName, iniPortSetS, PortSetS);
    WriteInteger(SectName, iniCDWaitSec, CDWaitSec);
    WriteInteger(SectName, iniHandShakingType, HandShakingType);
    WriteString(SectName, iniPhone, Phone);
    WriteString(SectName, iniLoginPrompt, LoginPrompt);
    WriteString(SectName, iniPasswd, Passwd);
    WriteString(SectName, iniUserName, UserName);
    WriteString(SectName, iniManualLogStr, ManualLogStr);
    WriteString(SectName, iniReadyTransaction, ReadyTransactionStr);
    WriteInteger(SectName, iniOptions, SetAsLong(PgConOptSet, SizeOf(TPgConOptSet)));
  end;
  Ini.Free;
  Result:= True;
end;

procedure TFormConnectOptions.FormActivate(Sender: TObject);
begin
  {
  if DoLoad
  then LoadIni;
  }
  {       }
  FillFields(False);
end;

procedure TFormConnectOptions.FormCreate(Sender: TObject);
begin
  {    -
        ConnectPars
    FormActivate  FillFields     ConnectPars
  }
  FillChar(ConnectPars, SizeOf(TConnectPars), #0);
end;

procedure TFormConnectOptions.CBPhoneClick(Sender: TObject);
begin
  EPhone.Enabled:= CBPhone.Checked;
end;

procedure TFormConnectOptions.CBNoWaitReadyClick(Sender: TObject);
begin
  EReadyTransaction.Enabled:= not CBNoWaitReady.Checked;
end;

procedure TFormConnectOptions.OKBtnClick(Sender: TObject);
begin
  {  }
  FillFields(True);
  {
  if DoStore
  then StoreIni;
  }
end;

procedure TFormConnectOptions.CBManualLogonClick(Sender: TObject);
begin
  if CBManualLogon.Checked then begin
     LAutoLogonEnabled.Caption:= '& -   ';
     {  control  :
      CBWaitPrompt
      ELoginPrompt
      EId.Enabled
      EPasswd
     }
     TSAutoLogon.Enabled:= False;
     EMManualLogon.Enabled:= True;
   end else begin
     LAutoLogonEnabled.Caption:= '&';
     TSAutoLogon.Enabled:= True;
     EMManualLogon.Enabled:= False;
   end;
end;

procedure TFormConnectOptions.StoreIni;
begin
  ConStoreIni('', ConnectPars);
end;

procedure TFormConnectOptions.LoadIni;
begin
  {          }
  ConLoadIni('', ConnectPars);
end;

procedure TFormConnectOptions.Button1Click(Sender: TObject);
var
  CC: TCommConfig;
  Buf: array[0..7] of Char;
begin
  if CommConfigDialog(StrPCopy(Buf, 'COM' + EPort.Text), 0, CC) then begin
    with CC.dcb do begin
      //'baud=1200 parity=N data=8 stop=1'
      // or 1200,N,8,1 or 12,N,8,1
      EPars.Text:= IntToStr(BaudRate)+','+
        ParityChar(Parity) +','+
        IntToStr(ByteSize) +','+
        IntToStr(StopBits);
      { wrong bit field
      if (Flags and PCF_XONXOFF) = XONXOFF
      then RGHandShaking.ItemIndex:= 1
      else RGHandShaking.ItemIndex:= 0;
      if (Flags and PCF_RTSCTS) = RTSCTS
      then RGHandShaking.ItemIndex:= RGHandShaking.ItemIndex + 2;
      }
    end;
    { call SetCommConfig }
  end;
end;


procedure TFormConnectOptions.CBFlowClick(Sender: TObject);
begin
  {}
  GBFlowChar.Visible:= CBFlowXOnXOff.Checked;
  CBFlowNone.Checked:= not (CBFlowXONXOff.Checked or
    CBFlowRTSCTS.Checked or CBFlowDTRDSR.Checked);
end;

procedure TFormConnectOptions.CBFlowNoneClick(Sender: TObject);
begin
  if CBFlowNone.Checked then begin
    CBFlowXONXOff.Checked:= False;
    CBFlowRTSCTS.Checked:= False;
    CBFlowDTRDSR.Checked:= False;

    GBFlowChar.Visible:= False;
  end;
end;

end.
