/*
    SQL script create EPager database table. (c) 1998, A.Ivanov. All rights reserved.
 1) Create INTERBASE alias EPAGER as described below (see README.TXT 
    for more details)
    PATH=C:\Src\Delphi\Pager\Data\Pager.gdb (example path)
    CODEPAGE=ParadoxCyr,Win1251 or similar (cyrillic code page 1251)
 2) Create Operator user with appropriate password (optional)
 3) change directory to "C:\Src\Delphi\Pager\Data" (select any other path)
 4) run this script via WISQL32 or ISQL (loacted in .\BIN subdirectory 
    of Interbase folder, where
    SQL.BAT 	-	interactive utility (if you want to see result)
    1.BAT	-	command line utility (if you sure)
*/

/* Change USER and PASSWORD clauses to appropriate values 
*/
SET NAMES WIN1251;
CREATE DATABASE "EPager.gdb"
USER "SYSDBA" PASSWORD "masterkey" DEFAULT CHARACTER SET WIN1251;
COMMIT;

CREATE TABLE DICT
 (PHRASE  VARCHAR(32)
 ,COUNTS  INTEGER
 );

CREATE TABLE VISITS
 (ID  INTEGER
 ,HCODE  VARCHAR(4)
 ,ABONENT  VARCHAR(6)
 ,EMPL  VARCHAR(12)
 ,UPDATER  INTEGER
 ,UPDATED  DATE
 ,UPDTIME  DATE
 ,ACTION  VARCHAR(64)
 );

CREATE TABLE VOC
 (LIST  VARCHAR(4)
 ,SUBLIST  VARCHAR(4)
 ,TEXT  VARCHAR(32)
 ,CODE  INTEGER
 ,REM  VARCHAR(32)
 );

CREATE TABLE CAPS
 (CAP  VARCHAR(12)
 ,BCAST  VARCHAR(12)
 ,CHAN INTEGER
 );

CREATE TABLE SERVICE
 (ID INTEGER
 ,HCODE VARCHAR(4)
 ,ABONENT VARCHAR(6)
 ,SERVICE INTEGER
 ,DSTART DATE
 ,DFINISH DATE
 );

CREATE TABLE SERVDESC
 (SERVICE INTEGER
 ,DESCRIPTION VARCHAR(64)
 ,MEASURE VARCHAR(4)
 ,PRICE FLOAT(126)
 ,CAP VARCHAR(12)
 ,CHAN INTEGER
 );

CREATE TABLE MODELS
 (PGMODEL  VARCHAR(16)
 ,LEN  INTEGER
 ,PROTOCOL  VARCHAR(12)
 ,SPEED  INTEGER
 ,CHARSET  VARCHAR(12)
 ,PRICE  FLOAT(126)
 );

CREATE TABLE CALLS
 (ID  INTEGER
 ,CALLGRP VARCHAR(1)
 ,HCODE  VARCHAR(4)
 ,ABONENT  VARCHAR(6)
 ,UPDATER INTEGER
 ,UPDATED DATE
 ,UPDTIME DATE
 ,MSGNO INTEGER
 ,SUBJ VARCHAR(24)
 ,MSG VARCHAR(255)
 ,CHARS INTEGER
 ,WORDS INTEGER
 ,ATTACH VARCHAR(64)
 ,ZONE  VARCHAR(4)
 ,GMT  VARCHAR(1)
 ,SSHEDULE VARCHAR(1)
 ,SDATE DATE
 ,SSTART  DATE
 ,SREPEAT VARCHAR(1)
 ,SFINISH  DATE
 ,SRPTMIN  INTEGER
 ,REPEATED INTEGER
 ,SKIND  VARCHAR(1)
 ,MAILED  VARCHAR(1)
 ,DELIVERED  VARCHAR(1)
 );

CREATE TABLE ZONES
 (ZONE  VARCHAR(4)
 ,DESCRIPTION  VARCHAR(32)
 ,EMAIL  VARCHAR(128)
 ,INSERVICE  VARCHAR(1)
 );

CREATE TABLE GROUPS
 (ID INTEGER
 ,GROUPID INTEGER
 ,HCODE VARCHAR(4)
 ,ABONENT VARCHAR(6)
 );

CREATE TABLE GRPDESC
 (GROUPID INTEGER
 ,NICKNAME VARCHAR(32)
 ,GRPADMIN VARCHAR(6)
 ,GCAP VARCHAR(12)
 ,GCHAN INTEGER
 ,DESCRIPTION VARCHAR(64)
 ,PASSWD VARCHAR(32)
 );

CREATE TABLE PAGERS
 (ID INTEGER
 ,CAP VARCHAR (12)
 ,PGMODEL  VARCHAR(16)
 ,PGSN  VARCHAR(16)
 ,PGDATE  DATE
 ,PGRESELLER  VARCHAR(16)
 ,PGWARRANTY  DATE
 ,INREPAIR  VARCHAR(1)
 ,REPAIRFROM  DATE
 ,REPAIRMAN  VARCHAR(12)
 ,LASTDEFECT  VARCHAR(12)
 ,ABONENT VARCHAR(6)
 ,REM VARCHAR(1)
 );

CREATE TABLE CLIENT
 (HCODE  VARCHAR(4)
 ,ABONENT  VARCHAR(6)
 ,CAP  VARCHAR(12)
 ,FAM  VARCHAR(25)
 ,IM  VARCHAR(25)
 ,NICKNAME VARCHAR(32)
 ,PASSWD VARCHAR(32)
 ,OT  VARCHAR(25)
 ,BIRTH  DATE
 ,SEX  VARCHAR(1)
 ,BUSINESS  VARCHAR(20)
 ,SERVICECAT  VARCHAR(12)
 ,DOCNAME  VARCHAR(20)
 ,DOCSER  VARCHAR(6)
 ,DOCNO  VARCHAR(10)
 ,DOCDATE  DATE
 ,DOCKEM  VARCHAR(25)
 ,PHONE  VARCHAR(12)
 ,POST  VARCHAR(6)
 ,CITY  VARCHAR(12)
 ,STREET  VARCHAR(25)
 ,BUILDING  VARCHAR(30)
 ,LOCAL  VARCHAR(1)
 ,PHONET  VARCHAR(8)
 ,STREETT  VARCHAR(25)
 ,BUILDINGT  VARCHAR(30)
 ,ORG  VARCHAR(1)
 ,ORGNAME  VARCHAR(20)
 ,ORGNO  INTEGER
 ,BILLNAME  VARCHAR(25)
 ,BILLSER  VARCHAR(8)
 ,BILLDOG  VARCHAR(12)
 ,BILLNO  VARCHAR(10)
 ,INSERVICE  VARCHAR(1)
 ,SERVICEFROM  DATE
 ,SERVICETO  DATE
 ,CREATED  DATE
 ,UPDATER  INTEGER
 ,UPDATED  DATE
 ,UPDTIME  VARCHAR(11)
 ,ZONE  VARCHAR(4)
 );

CREATE TABLE PAYEE
 (ID  INTEGER
 ,HCODE  VARCHAR(4)
 ,ABONENT  INTEGER
 ,DATEFROM  DATE
 ,DATETO  DATE
 ,MORTGAGE  FLOAT(126)
 ,BILL  VARCHAR(12)
 ,CARD  VARCHAR(6)
 ,MSGS  INTEGER
 ,WORDS  INTEGER
 ,CHARS  INTEGER
 );

CREATE  UNIQUE INDEX VISITS0 ON VISITS
 (ID);

CREATE  INDEX IABONENT ON VISITS
 (HCODE
 ,ABONENT);

CREATE  UNIQUE INDEX VOC0 ON VOC
 (LIST
 ,SUBLIST
 ,TEXT);

CREATE UNIQUE INDEX DICT0 ON DICT
 (PHRASE
 );

CREATE  UNIQUE INDEX ICAP0 ON CAPS
 (CAP
 ,BCAST
 ,CHAN);

CREATE  INDEX IUPD ON CALLS
 (HCODE
 ,ABONENT
 ,UPDATED
 ,UPDTIME);

CREATE INDEX ITIME ON CALLS
 (UPDATED
 ,UPDTIME);

CREATE  UNIQUE INDEX CALLS0 ON CALLS
 (ID
 );

CREATE UNIQUE INDEX  PAGERS0 ON PAGERS
 (ID
 );

CREATE INDEX  IPCAP ON PAGERS
 (CAP
 );

CREATE INDEX  IPMODEL ON PAGERS
 (PGMODEL
 );

CREATE INDEX GNICKNAME ON GRPDESC
 (NICKNAME
 );

CREATE  INDEX ICORG ON CLIENT
 (ORGNAME
 ,ORGNO);

CREATE INDEX ICBILL ON CLIENT
 (BILLNAME
 ,BILLSER
 ,BILLDOG
 ,BILLNO);

CREATE INDEX ICNICKNAME ON CLIENT
 (NICKNAME
 );

CREATE  UNIQUE INDEX CLIENT0 ON CLIENT
 (HCODE
 ,ABONENT);

CREATE  UNIQUE INDEX SERVDESC0 ON SERVDESC
 (SERVICE
 );

CREATE  INDEX ICFIO ON CLIENT
 (FAM
 ,IM
 ,OT);

CREATE  INDEX ISAB ON SERVICE
 (HCODE
 ,ABONENT
 );

CREATE  UNIQUE INDEX GRPDESC0 ON GRPDESC
 (GROUPID
 );

CREATE  INDEX IGAB ON GROUPS
 (HCODE
 ,ABONENT);

CREATE INDEX IGRP ON GROUPS
 (GROUPID
 );

CREATE  UNIQUE INDEX PAYEE0 ON PAYEE
 (ID
 );

CREATE  INDEX IPAB ON PAYEE
 (HCODE
 ,ABONENT);

/* CALLS **************************
*/
CREATE GENERATOR CALLS_NO_GEN;
/* set first autoincremental ID INTEGER
*/
SET GENERATOR CALLS_NO_GEN TO 1;

SET TERM !! ;
CREATE TRIGGER SET_CALLS_NO FOR CALLS
BEFORE INSERT 
AS
BEGIN
  NEW.ID = GEN_ID(CALLS_NO_GEN, 1);
END !!
SET TERM ; !!

/* PAGERS **************************
*/
CREATE GENERATOR PAGERS_NO_GEN;
/* set first autoincremental ID INTEGER
*/
SET GENERATOR PAGERS_NO_GEN TO 1;

SET TERM !! ;
CREATE TRIGGER SET_PAGERS_NO FOR PAGERS
BEFORE INSERT 
AS
BEGIN
  NEW.ID = GEN_ID(PAGERS_NO_GEN, 1);
END !!
SET TERM ; !!


/* VISITS *************************
*/
CREATE GENERATOR VSS_NO_GEN;
/* set first autoincremental ID INTEGER
*/
SET GENERATOR VSS_NO_GEN TO 1;

SET TERM !! ;
CREATE TRIGGER SET_VSS_NO FOR VISITS
BEFORE INSERT 
AS
BEGIN
  NEW.ID = GEN_ID(VSS_NO_GEN, 1);
END !!
SET TERM ; !!

/* SERVICE *************************
*/
CREATE GENERATOR SERVICE_NO_GEN;
/* set first autoincremental ID INTEGER
*/
SET GENERATOR SERVICE_NO_GEN TO 1;

SET TERM !! ;
CREATE TRIGGER SET_SERVICE_NO FOR SERVICE
BEFORE INSERT 
AS
BEGIN
  NEW.ID = GEN_ID(SERVICE_NO_GEN, 1);
END !!
SET TERM ; !!

/* GROUPS *************************
*/
CREATE GENERATOR GROUPS_NO_GEN;
/* set first autoincremental ID INTEGER
*/
SET GENERATOR GROUPS_NO_GEN TO 1;

SET TERM !! ;
CREATE TRIGGER SET_GROUPS_NO FOR GROUPS
BEFORE INSERT 
AS
BEGIN
  NEW.ID = GEN_ID(GROUPS_NO_GEN, 1);
END !!
SET TERM ; !!

COMMIT;
