unit Voices;
(*##*)
(*******************************************************************
*                                                                 *
*   V  O  I  C  E  S           EPAGER's unit.                      *
*                                                                 *
*   Copyright (c) 1998, ..           *
*                                                                 *
*   Conditioonal defines: ---                                      *
*                                                                 *
*   Last Revision: Apr 15 1998                                     *
*   Last fix     : Apr 15 1998                                    *
*   Lines        : 1641                                            *
*   History      : see CHANGES.TXT file                           *
*   Printed      : ---                                             *
*                                                                 *
********************************************************************)
(*##*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, MPlayer, Menus, Grids, ExtCtrls;

type
  TFormVoices = class(TForm)
    MediaPlayer1: TMediaPlayer;
    ListView1: TListView;
    PopupMenu1: TPopupMenu;
    popAdd: TMenuItem;
    popStore: TMenuItem;
    popLoad: TMenuItem;
    popD1: TMenuItem;
    SGProperties: TStringGrid;
    popDelete: TMenuItem;
    ImageList1: TImageList;
    popD2: TMenuItem;
    popShowProperties: TMenuItem;
    Splitter1: TSplitter;
    popEdit: TMenuItem;
    popEditVoice: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure popAddClick(Sender: TObject);
    procedure ListView1Click(Sender: TObject);
    procedure popLoadClick(Sender: TObject);
    procedure popStoreClick(Sender: TObject);
    procedure SGPropertiesExit(Sender: TObject);
    procedure popDeleteClick(Sender: TObject);
    procedure SGPropertiesClick(Sender: TObject);
    procedure popShowPropertiesClick(Sender: TObject);
    procedure popEditClick(Sender: TObject);
    procedure popEditVoiceClick(Sender: TObject);
  private
    { Private declarations }
    function ChooseVoiceFile(var AFN: String): Boolean;
  public
    { Public declarations }
    function  LoadList(AFN: String): Boolean;
    procedure StoreList(AFN: String);
    procedure ClearList;
    procedure UpdateProperties(AStore: Boolean);
  end;

var
  FormVoices: TFormVoices;

const
  FormVoicesOnScreen: Boolean = False;
  VoiceActive: Boolean = False;
  VoicesInMemory: Boolean = False;
  VoicesMaxInMemory: Word = 5;
  VoiceFolder: String[65] = '';
  VoiceListFN: String[65] = '';

implementation

uses
  util1, pcommon, DModule;

{$R *.DFM}

procedure TFormVoices.FormCreate(Sender: TObject);
begin
  Voices.FormVoicesOnScreen:= True;
  if (VoiceListFN <>'') and FileExists(ConCatPath(VoiceFolder, VoiceListFN)) then begin
    if LoadList(ConCatPath(VoiceFolder, VoiceListFN)) then begin
      FormVoices.Caption:= ' : ' + VoiceListFN;
    end;
  end else begin
    FormVoices.Caption:= ' :  ';
  end;
  SGProperties.Cells[0, 0]:= '';
  SGProperties.Cells[1, 0]:= '';
  SGProperties.Cells[2, 0]:= '';
  // ListView1.FullExpand;
end;

procedure TFormVoices.FormDestroy(Sender: TObject);
begin
  Voices.FormVoicesOnScreen:= False;
end;

procedure TFormVoices.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Free;
end;

function TFormVoices.ChooseVoiceFile(var AFN: String): Boolean;
begin
  with DataModule1.OpenDialog1 do begin
    InitialDir:= VoiceFolder;
    FileName:= ExtractFileName(AFN);
    DefaultExt:= 'TXT';
    Filter:= ' (*.WAV)|*.WAV|MIDI (*.MID)|*.MID|  (*.*)|*.*';
    FilterIndex:= 1;
    Title:= '  ';
    if Execute then begin
      AFN:= FileName;
      Result:= True;
    end else begin
      Result:= False;
    end;
  end;
end;

{   := , ,  }
procedure TFormVoices.popAddClick(Sender: TObject);
var
  L: TListItem;
  S: String;
begin
  S:= '';
  if ChooseVoiceFile(S) then begin
    L:= ListView1.Items.Add;
    L.Caption:= '  '+DateToStr(Date);
    L.SubItems.Add('-');
    L.SubItems.Add(S);
    L.SubItems.Add('9');
  end;
end;

procedure TFormVoices.ListView1Click(Sender: TObject);
begin
  { := , ,  }
  with ListView1 do begin
    if (Selected <> Nil) and ( Selected.SubItems.Count >= 3) and
      (Selected.SubItems[1]<> '')
    then begin
      MediaPlayer1.FileName:= Selected.SubItems[1];
      try
        MediaPlayer1.Open;
        MediaPlayer1.Play;
      except
        ShowMessage('     "'+
          Selected.Caption+'".');
      end;
      UpdateProperties(False);
    end;
  end;
end;

{   }
function TFormVoices. LoadList(AFN: String): Boolean;
var
  T: TextFile;
  S: String;
  i, j: Integer;
  L: TListItem;
begin
  Result:= False;
  if FileExists (AFN) then begin
    AssignFile(T, AFN);
    Reset(T);
    ListView1.Items.Clear;
    i:= 0;
    while not EOF(T) do begin
      Readln(T, S);
      L:= ListView1.Items.Add;
      L.Caption:= GetToken(1, '<P>', S);
      for j:= 0 to 2 do begin
        L.SubItems.Add(GetToken(j + 2, '<P>', S));
      end;
      if L.SubItems[1] = '' then begin
        Inc(i);
        if i > MAXEMPTYVOICES then begin
          ShowMessage('      ');
          Break;
        end;
      end;
    end;
    CloseFile(T);
    Result:= True;
  end;
end;

procedure TFormVoices.popLoadClick(Sender: TObject);
begin
  {   }
  with DataModule1.OpenDialog1 do begin
    // InitialDir:= VoiceFolder;
    DefaultExt:= 'TXT';
    FileName:= VoiceListFN;
    InitialDir:= VoiceFolder;
    Filter:= '  (*.TXT)|*.TXT|  (*.*)|*.*';
    FilterIndex:= 1;
    Title:= '  ';
    if Execute then begin
      if LoadList(FileName) then begin
        VoiceListFN:= ExtractFileName(FileName);
        VoiceFolder:= ExtractFilePath(FileName);
      end;
    end;
  end;
end;

{   }
procedure TFormVoices.StoreList(AFN: String);
var
  i, j: Integer;
  T: TextFile;
begin
  AssignFile(T, AFN);
  Rewrite(T);
  for i:= 0 to ListView1.Items.Count - 1 do begin
    Write(T, ListView1.Items[i].Caption +'<P>');
    for j:= 0 to ListView1.Items[i].Subitems.Count-1 do begin
      Write(T, ListView1.Items[i].SubItems[j]+'<P>');
    end;
    Writeln(T);
  end;
  CloseFile(T);
end;

{       }
procedure TFormVoices.ClearList;
begin
  ListView1.Items.Clear;
  SGProperties.Rows[1].Clear;
  SGProperties.Enabled:= False;
end;

procedure TFormVoices.popStoreClick(Sender: TObject);
begin
  {   }
    {   := , ,  }
  with DataModule1.SaveDialog1 do begin
    // InitialDir:= VoiceFolder;
    DefaultExt:= 'TXT';
    FileName:= VoiceListFN;
    InitialDir:= VoiceFolder;
    Filter:= '  (*.TXT)|*.TXT|  (*.*)|*.*';
    FilterIndex:= 1;
    Title:= '  ';
    if Execute then begin
      VoiceListFN:= ExtractFileName(FileName);
      VoiceFolder:= ExtractFilePath(FileName);
      StoreList(ConCatPath(VoiceFolder, VoiceListFN));
    end;
  end;
end;

procedure TFormVoices.SGPropertiesExit(Sender: TObject);
begin
  UpdateProperties(True);
end;

procedure TFormVoices.UpdateProperties(AStore: Boolean);
var
  i: Integer;
begin
  with ListView1, SGProperties do begin
    if AStore then begin
      if Selected <> Nil then begin
        Selected.SubItems.Clear;
        for i:=  0 to 2 do begin
          Selected.SubItems.Add(Cells[i, 1]);
        end;
      end;
    end else begin
      if (Selected <> Nil) and (Selected.SubItems.Count >= 3) then begin
        //       
        SGProperties.Enabled:= True;
        for i:=  0 to 2 do begin
          Cells[i, 1]:= Selected.SubItems[i];
        end;
      end;
    end;
  end;
end;

procedure TFormVoices.popDeleteClick(Sender: TObject);
begin
  if ListView1.Selected = Nil
  then Exit;
  ListView1.Items.Delete(ListView1.Selected.Index);
  SGProperties.Rows[1].Clear;
  SGProperties.Enabled:= False;
end;

{      }
procedure TFormVoices.SGPropertiesClick(Sender: TObject);
var
  S: String;
begin
  S:= '';
  if (SGProperties.Selection.Left = 1) and (ListView1.Selected<> Nil)
    and (ListView1.Selected.SubItems.Count>=3) and ChooseVoiceFile(S) then begin
    ListView1.Selected.SubItems[1]:= S;
  end;  
end;

procedure TFormVoices.popShowPropertiesClick(Sender: TObject);
begin
  popShowProperties.Checked:= not popShowProperties.Checked;
  SGProperties.Visible:= popShowProperties.Checked;
end;

procedure TFormVoices.popEditClick(Sender: TObject);
begin
  {   }
  StoreList(ConCatPath(VoiceFolder, VoiceListFN));
  //  
  ClearList;
  //     
  EExecuteFile(ConCatPath(VoiceFolder, VoiceListFN));
end;

procedure TFormVoices.popEditVoiceClick(Sender: TObject);
var
  S: String;
begin
  {   }
  if (ListView1.Selected <> Nil) and (ListView1.Selected.SubItems.Count >= 3) then begin
    S:= ListView1.Selected.SubItems[1];
    if S <> '' then begin
      EExecuteFile(S);
    end;  
  end;
end;

end.
