unit LClient;
(*##*)
(*******************************************************************
*                                                                 *
*   L  C  L  I  E  N  T        tcptest  unit.                      *
*                                                                 *
*   Copyright (c) 1998, ..           *
*                                                                 *
*   Conditional defines: ---                                       *
*                                                                 *
*   Last Revision: Jun    1998                                     *
*   Last fix     : Jul 03 1998                                    *
*   Lines        : 1166                                            *
*   History      : see CHANGES.TXT file                           *
*   Printed      : ---                                             *
*                                                                 *
********************************************************************)
(*##*)

interface

uses
  Windows, Messages, SysUtils, Classes;

type
  URLString = String[80];
  TClientState= set of (csReady2Delete);
  TConnectPhase= (phConnected, phRegistration, phWorkflow, phDisconnected);
  TClient = record
    //     
    handle: Integer;
    connected: TDateTime;
    //     
    dt: TDateTime;
    //    
    regtryes: Integer;   // -  
    Enabled: Boolean;    //   
    Name  :  String[32]; //  
    host: URLString;
    ip: string[16];
    Phase: TConnectPhase;// 
    State: TClientState;
  end;

  TAClients = array[0..9] of TClient; // 9  -   
  PAClients = ^TAClients;

  TClients = class (TPersistent)
    private
    FClients: PAClients;
    FCount, FLimit, FStep: Integer;
    FTimeout,
    FNoMoreTime: TDateTime; // - 
    function getnobyhandle(Ahandle: Integer; var no: Integer): Boolean;
    function GetClientByOrder(no: Integer): TClient;
    procedure SetClientByOrder(no: Integer; AClient: TClient);
    function GetClient(Ahandle: Integer): TClient;
    procedure SetClient(Ahandle: Integer; AClient: TClient);
    procedure SetNewSize(ACount: Integer);
    procedure CheckSpace;
    procedure Add(Ahandle: Integer);
    function CreateEmptyClient: TClient;
    public
    RegTryesLimit: Integer;
    property TimeOut: TDateTime read FTimeout write FTimeOut;
    property Count: Integer read FCount write SetNewSize; //  
    property Clients[no: Integer]: TClient read GetClient write SetClient;
    property ClientsByOrder[no: Integer]: TClient read GetClientByOrder write SetClientByOrder;
    constructor Create;
    destructor  Destroy; override;
    procedure DeleteByNo(no: Integer);
    procedure Delete(Ahandle: Integer);
    procedure MarkTimeOut(what: TClientState);
    procedure DeleteMarked;
  end;

function PhaseString(AClient: TClient): String;

implementation

function PhaseString(AClient: TClient): String;
var
  S: String;
begin
  case AClient.Phase of
  phConnected:
    S:= '';
  phRegistration:
    S:= ':'+IntToStr(AClient.regtryes);
  phWorkflow:
    S:= '...';
  phDisconnected:
    S:= '';
  else
    S:= '???';
  end;
  Result:= S;
end;

constructor TClients.Create;
begin
  inherited Create;
  FCount:= 0;
  FStep:= 5;
  FTimeOut:= -1;
  FNoMoreTime:= -1;
  FClients:= Nil;
  RegTryesLimit:= 3;
  SetNewSize(0);
end;

destructor  TClients.Destroy;
begin
  SetNewSize(0);
  inherited Destroy;
end;

function    TClients.CreateEmptyClient: TClient;
var
  NewClient: TClient;
begin
  FillChar(NewClient, SizeOf(TClient), 0);
  Result:= NewClient;
end;

procedure   TClients.SetNewSize(ACount: Integer);
begin
  //  -    ?
  ReAllocMem(FClients, ACount*SizeOf(TClient));  //       
  //     
  if ACount > FLimit then begin
    FillChar(FClients^[FLimit], SizeOf(TClient)*(ACount-FLimit), 0);
  end;
  //   
  FLimit:= ACount;
  //     
  if ACount = 0 then begin
    FClients:= Nil;
  end;
end;

//   FCount
procedure TClients.CheckSpace;
begin
  //  -    ?
  if FCount >= Flimit
  then SetNewSize(FLimit + FStep)
  else begin
    if FCount + FStep < Flimit
    then SetNewSize(FLimit - FStep)
  end;
end;

//    handle
function    TClients.getnobyhandle(Ahandle: Integer; var no: Integer): Boolean;
var
  i: Integer;
begin
  for i:= 0 to FCount - 1 do begin
    if FClients^[i].handle = Ahandle then begin
      Result:= True;
      no:= i;
      Exit;
    end;
  end;
  Result:= False;
end;

function    TClients.GetClientByOrder(no: Integer): TClient;
begin
  if (no >= 0) and (no < FCount)
  then Result:= FClients^[no]
  else Result:= CreateEmptyClient;
end;

function    TClients.GetClient(Ahandle: Integer): TClient;
var
  no: Integer;
begin
  if getnobyhandle(AHandle, no)
  then Result:= FClients^[no]
  else Result:= CreateEmptyClient;
end;

procedure   TClients.Add(Ahandle: Integer);
begin
  // if getnobyhandle(AHandle, no)
  // then Exit;
  Inc(FCount);
  CheckSpace;
  with FClients^[FCount-1] do begin
    handle:= AHandle;
    regtryes:= 0;
    // dt:= Date + Time;
  end;
end;

procedure   TClients.DeleteByNo(no: Integer);
var
  shift: Integer;
begin
  if (no < FCount) and (no >= 0) then begin
    shift:= FCount - 1 - no;
    if shift > 0
    then Move(FClients^[no+1], FClients^[no], shift * SizeOf(TClient));
    Dec(FCount);
    CheckSpace;
  end;
end;

procedure   TClients.Delete(Ahandle: Integer);
var
  no: Integer;
begin
  if getnobyhandle(AHandle, no) then begin
    DeleteByNo(no);
  end;
end;

procedure   TClients.MarkTimeOut(what: TClientState);
var
  no: Integer;
  curdt: TDateTime;
begin
  curdt:= Date + Time;
  no:= 0;
  while no < FCount do begin
    if FClients^[no].dt + ftimeout < curdt then begin
      FClients^[no].State:= what
    end else begin
      FClients^[no].State:= [];
    end;
    Inc(no);
  end;
end;

procedure   TClients.DeleteMarked;
var
  no: Integer;
begin
  no:= 0;
  while no < FCount do begin
    if csReady2Delete in FClients^[no].State
    then DeleteByNo(no)
    else Inc(no);
  end;
end;

procedure   TClients.SetClientByOrder(no: Integer; AClient: TClient);
begin
  if (no >= 0) and (no < FCount) then begin
    FClients^[no]:= AClient;
    FClients^[no].Dt:= Date+Time;
  end;
end;

procedure   TClients.SetClient(Ahandle: Integer; AClient: TClient);
var
  no: Integer;
  curDt: TDateTime;
begin
  curdt:= Date+Time;
  if getnobyhandle(AHandle, no) then begin
    FClients^[no]:= AClient;
    //    
    FClients^[no].Handle:= AHandle;
  end else begin
    //     
    Add(Ahandle);
    if getnobyhandle(AHandle, no) then begin
      FClients^[no]:= AClient;
      //    
      FClients^[no].Handle:= AHandle;
      //   
      FClients^[no].Connected:= curdt;
    end;
  end;
  FClients^[no].Dt:= curdt;
end;

end.
