unit ComShape;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TComStateShape = class(TShape)
  private
    { Private declarations }
    FActiveColor: TColor;
    FInActiveColor: TColor;
    FPowerOffColor: TColor;
    FPowerOn: Boolean;
    FLightOn: Boolean;
    procedure SetActiveColor(AColor: TColor);
    procedure SetInActiveColor(AColor: TColor);
    procedure SetPowerOffColor(AColor: TColor);
    procedure SetPowerOnOff(AOn: Boolean);
    procedure SetLigthOnOff(AOn: Boolean);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property PowerOn: Boolean read FPowerOn write SetPowerOnOff default false;
    property LightOn: Boolean read FLightOn write SetLigthOnOff default false;
    property ActiveColor: TColor read FActiveColor write SetActiveColor default clNavy;
    property InActiveColor: TColor read FInActiveColor write SetInActiveColor default clLime;
    property PowerOffColor: TColor read FPowerOffColor write SetPowerOffColor default clWhite;
  end;

procedure Register;

implementation

constructor TComStateShape. Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetActiveColor(clLime);
  SetInActiveColor(clTeal);
  SetPowerOffColor(clSilver);
  SetPowerOnOff(False);
  Height:= 12;
  Width:= 12;
  Shape:= stCircle;
end;

procedure TComStateShape. SetActiveColor(AColor: TColor);
begin
  FActiveColor:= AColor;
  if FPowerOn and FLightOn
  then Brush.Color:= AColor;
end;

procedure TComStateShape. SetInActiveColor(AColor: TColor);
begin
  FInActiveColor:= AColor;
  if FPowerOn and (not FLightOn)
  then Brush.Color:= AColor;
end;

procedure TComStateShape. SetPowerOffColor(AColor: TColor);
begin
  FPowerOffColor:= AColor;
  if not FPowerOn
  then Brush.Color:= AColor;
end;

procedure TComStateShape. SetPowerOnOff(AOn: Boolean);
begin
  FPowerOn:= AOn;
  FLightOn:= False;
  if AOn and (not FLightOn) then begin
    Brush.Color:= FInActiveColor;
  end else begin
    Brush.Color:= FPowerOffColor;
  end;
end;

procedure TComStateShape. SetLigthOnOff(AOn: Boolean);
begin
  FPowerOn:= True;
  FLightOn:= AOn;
  if AOn then begin
    Brush.Color:= FActiveColor;
  end else begin
    Brush.Color:= FInActiveColor;
  end;
end;

procedure Register;
begin
  RegisterComponents('Ensen', [TComStateShape]);
end;

end.
