unit wmail;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, OleCtrls,
  ISP3, ToolWin, MSMAPI, Grids;

const
  // header stamp consts
  MSG_IDTYPE = 'Ensen smtp mail';
  MSG_CONTENTTYPE = 'TEXT/PLAIN charset=US-ASCII';
type

  TFormMail = class(TForm)
    POPStatus: TStatusBar;
    SGMsg: TStringGrid;
    HeaderControl1: THeaderControl;
    Splitter1: TSplitter;
    Memo1: TMemo;
    CoolBar1: TCoolBar;
    ToolBar1: TToolBar;
    TBSignOn: TToolButton;
    TBSignOff: TToolButton;
    MainMenu1: TMainMenu;
    MFile: TMenuItem;
    MMessages: TMenuItem;
    MOptions: TMenuItem;
    MOptionsUnreadOnly: TMenuItem;
    N1: TMenuItem;
    MFileExit: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TBSignOnClick(Sender: TObject);
    procedure TBSignOffClick(Sender: TObject);
    procedure CBUnreadOnlyClick(Sender: TObject);
    procedure SGMsgClick(Sender: TObject);
    procedure HeaderControl1SectionResize(HeaderControl: THeaderControl;
      Section: THeaderSection);
    procedure MFileExitClick(Sender: TObject);
    procedure MOptionsUnreadOnlyClick(Sender: TObject);
  private
    RecvVerified,
    SMTPError,
    POPError: Boolean;
    FMessageCount: Integer;
  public
    SessionEstablished: Boolean;
    MailSlot: Integer;
    MsgCount: Integer;
    procedure FetchMessageList;
  end;

var
  FormMail: TFormMail;
 
implementation

{$R *.DFM}

uses OLEAuto;

const 
  icDocBegin = 1;
  icDocHeaders = 2;
  icDocData = 3;
  icDocEnd = 5;

{When calling a component method which maps onto an OLE call, NoParam substitutes
for an optional parameter. As an alternative to calling the component method, you
may access the component's OLEObject directly - 
i.e., Component.OLEObject.MethodName(,Foo,,Bar)}
function NoParam: Variant;
begin
  TVarData(Result).VType := varError;
  TVarData(Result).VError := DISP_E_PARAMNOTFOUND;
end;

procedure TFormMail.FormCreate(Sender: TObject);
begin
  SMTPError := False;
  POPError := False;
  FMessageCount := 0;
  SessionEstablished:= False;
end;

procedure TFormMail.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  TBSignOffClick(Self);
end;

{The RefreshMessageCount event is called whenever the RefreshMessageCount method is
called, and also when a connection to the POP server is first made}
procedure TFormMail.TBSignOnClick(Sender: TObject);
const
   MailSlotName = '\\.\mailslot\sample_mailslot';
var
  MaxMessageSize,        // address of maximum message size
  NextSize,              // address of size of next message
  MessageCount,          // address of number of messages
  ReadTimeout: Integer;  // address of read time-out

begin
  if SessionEstablished then begin
    CloseHandle(MailSlot);
  end;
  MailSlot:= CreateMailslot(MailSlotName,
    0,                                    // no maximum message size
    MAILSLOT_WAIT_FOREVER,                // no time-out for read operations
    Nil);                                 // no security attributes
  if MailSlot = INVALID_HANDLE_VALUE then begin
    ShowMessage('CreateMailslot');
    Exit;
  end;
  if GetMailslotInfo(
    MailSlot,
    @MaxMessageSize,	// address of maximum message size
    NextSize,           // address of size of next message
    @MessageCount,	// address of number of messages
    @ReadTimeout 	// address of read time-out
   ) then;
  SessionEstablished:= True;
  PopStatus.Panels[2].Text:= IntToStr(MailSlot)+
  ' . :'+IntToStr(MaxMessageSize)+
  '  ..:'+IntToStr(MaxMessageSize)+
  ' :'+IntToStr(MaxMessageSize)+
  ' -:'+IntToStr(MaxMessageSize);
  FetchMessageList;
end;

procedure TFormMail.TBSignOffClick(Sender: TObject);
begin
  if SessionEstablished then begin
    CloseHandle(MailSlot);
  end;
  SessionEstablished:= False;
  PopStatus.Panels[2].Text:= ' ';
  PopStatus.Panels[0].Text:= ': 0';
end;

procedure TFormMail.CBUnreadOnlyClick(Sender: TObject);
begin
  FetchMessageList;
end;

procedure TFormMail.FetchMessageList;
var
  col, row: Integer;
begin
  if not SessionEstablished
  then Exit;
  // MMsg.FetchUnreadOnly:= MOptionsUnreadOnly.Checked;
  // MMsg.Fetch;
  // PopStatus.Panels[0].Text:= ': '+IntToStr(MMsg.MsgCount);
  {if MMsg.MsgCount = 0 then begin
    SGMsg.RowCount:= 1;
    for col:= 1 to SGMsg.ColCount
    do SGMsg.Cells[col-1, 0]:= '';
    Exit;
  end;
  SGMsg.RowCount:= MMsg.MsgCount;
  for row:= 1 to SGMsg.RowCount do begin
    MMsg.MsgIndex:= row - 1;
    SGMsg.Cells[0, row-1]:= MMsg.MsgSubject;
    SGMsg.Cells[1, row-1]:= MMsg.MsgNoteText;
    SGMsg.Cells[2, row-1]:= MMsg.MsgDateReceived;
    SGMsg.Cells[3, row-1]:= MMsg.MsgType;
    SGMsg.Cells[4, row-1]:= MMsg.AttachmentName;
  end;
  }
end;

procedure TFormMail.SGMsgClick(Sender: TObject);
begin
  {    }
  if SGMsg.Selection.Top + 1 <= MsgCount then begin
    {        }
    // MMsg.MsgIndex:= SGMsg.Selection.Top;
    {    }
    // Memo1.Text:= MMsg.MsgNoteText;
  end;
end;

procedure TFormMail.HeaderControl1SectionResize(
  HeaderControl: THeaderControl; Section: THeaderSection);
var
  i: Integer;
begin
  {}
  for i:= 1 to SgMsg.ColCount do begin
    SgMsg.ColWidths[i-1]:= HeaderControl1.Sections[i-1].Width - 1;
  end;
  SgMsg.ColWidths[0]:= SgMsg.ColWidths[0] - 1;
end;

procedure TFormMail.MFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMail.MOptionsUnreadOnlyClick(Sender: TObject);
begin
  MOptionsUnreadOnly.Checked:= not MOptionsUnreadOnly.Checked;
end;

end .
