unit
  Osetting;
(*##*)
(*******************************************************************
*                                                                 *
*   O  S  E  T  T  I  N  G     EPAGER's unit.                      *
*                                                                 *
*   Copyright (c) 1998, ..           *
*   common EPAGER settings form (assign alias, font, etc)         *
*   Conditional defines: ---                                       *
*                                                                 *
*   Last Revision: Mar 19 1998                                     *
*   Last fix     : Mar 19 1998                                    *
*   Lines        : 218                                             *
*   History      : see CHANGES.TXT file                           *
*   Printed      : ---                                             *
*                                                                 *
********************************************************************)
(*##*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Dialogs, IniFiles, db, DBTables, SysUtils,
  util1, pcommon;

type
  TFormSettings = class(TForm)
    Label9: TLabel;
    EDBFolder: TEdit;
    Memo1: TMemo;
    Image5: TImage;
    GBoxAlias: TGroupBox;
    CBAlias: TComboBox;
    CBAliasCyr: TComboBox;
    RGByAlias: TRadioButton;
    RGByName: TRadioButton;
    BSetupPrinter: TButton;
    BFont: TButton;
    FontDialog1: TFontDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    BOk: TButton;
    BCancel: TButton;
    CBDbDriver: TComboBox;
    LDriverName: TLabel;
    BDbFolder: TButton;
    procedure BSetupPrinterClick(Sender: TObject);
    procedure BFontClick(Sender: TObject);
    procedure CBAliasCyrChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BCancelClick(Sender: TObject);
    procedure EDBFolderChange(Sender: TObject);
    procedure RGByNameClick(Sender: TObject);
    procedure RGByAliasClick(Sender: TObject);
    procedure BDbFolderClick(Sender: TObject);
  private
    { Private declarations }
    AliasCyrList: TStringList;
    procedure FillAliasList;
  public
    { Public declarations }
    FontChanged: Boolean;
    DBLocationChanged: Boolean;
    procedure Set2Alias(const AAlias, ADBFolder, ADBDriver: String;
      AUseAlias: Boolean; ADB: TDataBase);
  end;

var
  FormSettings: TFormSettings;

implementation

uses
  CDatas;

{$R *.DFM}

procedure TFormSettings.BSetupPrinterClick(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TFormSettings.BFontClick(Sender: TObject);
begin
  with FontDialog1 do if Execute then begin
    Self.Font:= Font;
    FontChanged:= True;
  end;
end;

procedure TFormSettings.FillAliasList;
var
  IniFile: TIniFile;
  S: ShortString;
  i: Integer;
begin
  IniFile:= TIniFile.Create(INI_PAGER_FILE);
  with IniFile do if AliasCyrList <> Nil then begin
    {  AliasCyrList   OnCreate }
    ReadSectionValues(sectALIASESCYR, AliasCyrList);
    {   combobox }
    CBAliasCyr.Clear;
    for i:= 1 to AliasCyrList.Count do begin
      S:= AliasCyrList.Strings[i-1];
      CBAliasCyr.Items.Add(System.Copy(S,Pos('=',S)+1,255));
    end;
    IniFile.Free;
  end;
end;

procedure TFormSettings.CBAliasCyrChange(Sender: TObject);
var
  i: Integer;
  S, SS: ShortString;
begin
  {  alias     }
  i:= CBAliasCyr.Items.IndexOf(CBAliasCyr.Text);
  if i < 0 then Exit;
  {              }
  S:= AliasCyrList.Strings[i];
  SS:= UpperCase(System.Copy(S,1, Pos('=',S)));
  {   '=' }
  System.Delete(SS, Length(SS), 1);
  {     }
  CBAlias.Text:= '';
  for i:= 1 to CBAlias.Items.Count do begin
    S:= Uppercase(CBAlias.Items[i-1]);
    if S = SS then begin
      CBAlias.Text:=CBAlias.Items[i-1];
      Break;
    end;
  end;
  {  CBAliasCyr.Text:= AliasCyrList.Values[CBAlias.Text];
  }
  DBLocationChanged:= True;
end;

procedure TFormSettings.FormCreate(Sender: TObject);
begin
  FontChanged:=False;
  DBLocationChanged:= False;
  {    .  ,   }
  AliasCyrList:= TStringList.Create;
end;

procedure TFormSettings.FormDestroy(Sender: TObject);
begin
  if AliasCyrList <> Nil
  then AliasCyrList.Free;
end;

procedure TFormSettings.BCancelClick(Sender: TObject);
begin
  FontChanged:=False;
  DBLocationChanged:= False;
end;

procedure TFormSettings.Set2Alias(const AAlias, ADBFolder, ADBDriver: String;
  AUseAlias: Boolean; ADB: TDataBase);
begin
  ADB.Session.GetAliasNames(CBAlias.Items);
  ADB.Session.GetDriverNames(CbDbDriver.Items);
  CBAlias.Text:= AAlias;
  CbDbDriver.Text:= ADbDriver;
  FillAliasList;
  CBAliasCyr.Text:= AliasCyrList.Values[AAlias];
  RGByAlias.Checked:= AUseAlias;
  RGByName.Checked:= not AUseAlias;  
  EDBFolder.Text:= ADBFolder;
end;

procedure TFormSettings.EDBFolderChange(Sender: TObject);
begin
  DBLocationChanged:= True;
end;

procedure TFormSettings.RGByNameClick(Sender: TObject);
begin
  DBLocationChanged:= True;
end;

procedure TFormSettings.RGByAliasClick(Sender: TObject);
begin
  DBLocationChanged:= True;
end;

procedure TFormSettings.BDbFolderClick(Sender: TObject);
begin
  {}
  with DataModule1.OpenDialog1 do begin
    InitialDir:= ExtractFilePath(EDBFolder.Text);
    FileName:= ExtractFileName(EDBFolder.Text);
    DefaultExt:= 'TXT';
    Filter:= ' (*.gdb)|*.gdb|  (*.*)|*.*';
    FilterIndex:= 1;
    Title:= '  ';
    if Execute then begin
      EDBFolder.Text:= FileName;
    end;
  end;
end;

end.
