unit AP_TAP;
(*##*)
(*******************************************************************
*                                                                 *
*   A  P  T  A  P         Async Pro TApdComPort component          *
*                                                                 *
*   interface                                 TAP implementation   *
*                                                                 *
*   Copyright (c) 1998, A.Ivanov. All rights reserved.             *
*   Delphi 2, 3 only (uses huge strings)                          *
*   Conditional defines: ASYNC H+                                  *
*   define MSCOMM if wants MSComm component                       *
*   Last Revision: Mar 19 1998                                     *
*   Last fix     : Mar 19 1998                                    *
*   Lines        : 431                                             *
*   History      : see CHANGES.TXT file                           *
*   Printed      : ---                                             *
*                                                                 *
********************************************************************)
(*##*)

interface
uses
  Classes, SysUtils, Windows, Forms, AdPort,
  util1, TAP;

type
  TSendProc = function(AMS: TApdComPort; const S: String): Boolean;
  TRecvFunc = function(AMS: TApdComPort): String;
  {  AP_SendLoop - callback     }
  TStateShowProc = procedure (const StateString: String);

  TAPDComPortArray = array[0..5] of TAPDComPort; //       [0..0]
  TAPDComPorts = ^TAPDComPortArray;
  //      tap.TTAPsPars    TAPDComPort
  //    
  TTAPTerminals = class (TTAPsPars)
    private
    FAPDComPorts: TAPDComPorts;
    function GetComPort(AIndex: Word): TAPDComPort;
    procedure SetComPort(AIndex: Word; AComPort: TAPDComPort);
    protected
    procedure SetTerminalPar(AIndex: Word; AValue: TTerminalPar);override;
    public
    constructor Create; override;
    procedure  SetCount(ACount: Word); override;
    property   ComPorts[index: Word]: TAPDComPort read GetComPort write SetComPort;
  end;

{      TApdComPort
}
{ .   TApdComPort  
}
function AP_SetComPort(AComm: TApdComPort; ConnectPars: TConnectPars): Boolean;

{     SWaitForB         TimeoutB  (  -  )
    SSend               TimeRepeat     
      SWaitForE         TimeoutE  (  -  )
}
function AP_TTYSend(AComm: TApdComPort; const SWaitForB, SSend, SWaitForE: String;
  TimeoutB, TimeRepeat, TimeoutE: Integer): Integer;

{ :
  TAPBlockLen -     TAP  ( 250)
  TAPLimitBlocks-     TAP  (0- .)
  MsgLen      -    ,  
}
function AP_SendLoop(AComm: TApdComPort; ATerminalPars: TTerminalPar;
  AGetMsgProc: TGetMsgProc;
  AMarkMsgProc: TMarkMsgProc;
  StateShow: TStateShowProc): Boolean;

implementation

uses
  ooMisc;

{  
}
function AP_SetComPort(AComm: TApdComPort; ConnectPars: TConnectPars): Boolean;
var
  p: Char;
begin
  with AComm do begin
    ComNumber:= ConnectPars.PortNo1;
    Baud:= StrToInt(GetToken(1, ',', ConnectPars.PortSetS));
    P:= Upcase(GetToken(2, ',', ConnectPars.PortSetS)[1]);
    DataBits:= StrToInt(GetToken(3, ',', ConnectPars.PortSetS));
    StopBits:= StrToInt(GetToken(4, ',', ConnectPars.PortSetS));
    case p of
    'E': Parity:= pEven;
    'M': Parity:= pMark;
    'N': Parity:= pNone;
    'O': Parity:= pOdd;
    'S': Parity:= pSpace;
    end;
    if ConnectPars.HandShakingType and 1 = 1
    then SWFlowOptions:= swfBoth
    else SWFlowOptions:= swfNone;
    HWFlowOptions:= [];
    if ConnectPars.HandShakingType and 2 = 2
    then HWFlowOptions:= HWFlowOptions + [hwfUseRTS, hwfRequireCTS];
    if ConnectPars.HandShakingType and 4 = 4
    then HWFlowOptions:= HWFlowOptions + [hwfUseDTR, hwfRequireDSR];
    Result:= True;
  end;
end;

{     SWaitForB         TimeoutB  (  -  )
    SSend               TimeRepeat      (  -  )
      SWaitForE         TimeoutE  (  -  )
   TimeRepeat = 0    SSend
  - 0      - OK  SWaitForE = NIL  
              1,2..  -     SWaitForE
              -1     - 
  SWaitForE -   ","  
}
function AP_TTYSend(AComm: TApdComPort;
  const SWaitForB, SSend, SWaitForE: String;
  TimeoutB, TimeRepeat, TimeoutE: Integer): Integer;
var
  r: Integer;
  Tim: Integer;
  ET: EventTimer;
begin
  Result:= -1;
  if SWaitForB <> '' then begin
    { ----------   ----------- }
    if not AComm.WaitForString(SWaitForB, Secs2Ticks(TimeOutB), True, True)
    then Exit;
  end;
  if TimeOutE > 0
  then NewTimerSecs(ET, TimeOutE);
  { --  -- }
  repeat
    { ----------  (   TimeRepeat > 0)----------- }
    if SSend <> '' then begin
      if AComm.OutBuffFree >= Length(SSend)
      then AComm.PutString(SSend)
      else Exit;
    end;
    if (TimeoutE = 0) or (SWaitForE = '') then begin
      {   , OK = true,   }
      Result:= 0;
      Exit;
    end;
    { ----------  -   ----------- }
    {  }
    if TimeRepeat > 0
    then Tim:= TimeRepeat
    else Tim:= TimeoutE;
    r:= AComm.WaitForMultiString(SWaitForE, Secs2Ticks(Tim),
      True, True, LISTDELIMITER);
    if r > 0 then begin
      Result:= r; //    (1..
      Exit;
    end;
    {        -,  -
          ,    
    }
  until (TimeRepeat=0) or (TimeOutE=0) or TimerExpired(ET);
end;

{ wait carrier detect signal at comm. port in SECS second
}
function AP_WaitState(AComm: TApdComPort; Secs: Integer; AMask: Byte): Boolean;
var
  ET: EventTimer;
begin
  Result:= False;
  NewTimerSecs(ET, Secs);
  repeat
    if AComm.ModemStatus and AMask = AMask then begin
      Result:= True;
      Exit;
    end;
    Application.ProcessMessages;
  until TimerExpired(ET);
end;

{ wait carrier detect signal at comm. port in SECS second
}
function AP_WaitCD(AComm: TApdComPort; Secs: Integer): Boolean;
begin
  Result:= AP_WaitState(AComm, Secs, DCDMask);
end;

{ prototypes:
  AP_TTYSend(SendProc: TSendProc; RecvFunc: TRecvFunc; const SWaitForB, SSend, SWaitForE: String;
  TimeoutB, TimeRepeat, TimeoutE: Integer): Boolean;
  APrompt -   ","  
}
function AP_WaitPrompt(AComm: TApdComPort; ConnectPars: TConnectPars; const ALog, APrompt: String;
  Secs: Integer; Rpt: Integer): Boolean;
begin
  if not (cmWaitPrompt in ConnectPars.PgConOptSet) then begin
    {     }
    Result:= True;
    Exit;
  end;
  Result:= AP_TTYSend(AComm, '', ALog, APrompt,0, Secs div Rpt, Secs) = 0;
end;

function  AP_CallModem(AComm: TApdComPort; const APhone: String; AWaitCD: Integer): Boolean;
begin
  AComm.Output:= WEBStyleAndControlString2ASCII(APhone);
  Result:= AP_WaitCD(AComm, AWaitCD);
end;

function  AP_TryLogon(AComm: TApdComPort; ConnectPars: TConnectPars; Secs: Integer;
  StateShow: TStateShowProc): Boolean;
var
  name_passwd: String;
  R: Integer;
begin
  if (cmManualLogin in ConnectPars.PgConOptSet) then begin
      {         }
    name_passwd:= WEBStyleAndControlString2ASCII(ConnectPars.ManualLogStr);
    { }
    StateShow(' -  ACK (NAK|EOT),    " "');
  end else begin
    {    "PG1" }
    name_passwd:= ESC + WEBStyleAndControlString2ASCII(ConnectPars.UserName);
    {        CR  ! }
    if ConnectPars.Passwd > ''
    then name_passwd:= name_passwd +
      WEBStyleAndControlString2ASCII(ConnectPars.Passwd) + CR;
  end;
  if (cmNoWaitAccept in ConnectPars.PgConOptSet) then begin
    {      }
    Secs:= 0;
  end;
  {  ACK-  NAK-   EOT-  }
  { :     }
  R:= AP_TTYSend(AComm, '', name_passwd, ACK+','+NAK+','+EOT, 0, 0, Secs);
// test R:= AP_TTYSend('', 'AT'#13#10, 'OK'+','+NAK+','+EOT, 0, 0, Secs);
  Result:= R = 0;
end;

function  AP_WaitReadyTransaction(AComm: TApdComPort; ConnectPars: TConnectPars;
  const AReadyString: String; Secs: Integer): Boolean;
begin
  if (cmNoWaitReady in ConnectPars.PgConOptSet) then begin
   {  ? }
    Result:= True;
  end else begin
    {   }
    Result:= AP_TTYSend(AComm,  AReadyString, '', '', Secs, 0, 0) = 0;
  end;
end;

{    
}
function AP_DisconnectTransaction(AComm: TApdComPort; Secs: Integer): Boolean;
begin
  {  ..  }
  {  ,     }
  Result:= AP_TTYSend(AComm, '', EOT+CR, ESC+EOT+CR+','+RS+CR,0, 0, Secs) = 0;
end;

const
  BLOCKSecs: Word = 10;
{    ABlock   AComm
        BLOCKSecs (T4= 10 )
  :
  0-   
  1- - ,   
  2-   CRC
  3-    
}
function AP_SendTAPBlock(AComm: TApdComPort; const ABlock: String): Word;
var
  R: Integer;
  CRCFaultCount: Word;
begin
  {      NAK }
  CRCFaultCount:= 0; //      MED  CRC  
  repeat
    {  }
    R:= AP_TTYSend(AComm, '', ABlock, ACK+','+NAK+','+EOT, 0, 0, BLOCKSecs);
    case R of
    -1:begin
       {   -  }
        Result:= 1;
        Exit;
      end;
    1: { ACK } begin
        Result:= 0;
        Exit;
      end;
    2: { NAK } begin
        {   7  (        ) }
        Inc(CRCFaultCount);
      end;
    3: { EOT } begin
        {   -    }
        Result:= 3;
        Exit;
      end;
    end; { case }
  until CRCFaultCount > 5;
  Result:= 2; // - 
end;

{    . AP_SendTAPBlock  AP_SendTAPBlocks
}
function ErrorSendBlockString(ACode: Integer): String;
begin
  case ACode of
  0: Result:= '  ';
  1: Result:= '- ,   ';
  2: Result:= '  CRC';
  3: Result:= '   ';
  end;
end;

{  ,  RS+CR
   RS CR ESC EOT CR - 
             ESC EOT CR -  
  Secs: 10     
  :
  True-       
  False-    ,  ()    
          :   ,...
}

function AP_SendLoop(AComm: TApdComPort; ATerminalPars: TTerminalPar;
  AGetMsgProc: TGetMsgProc;
  AMarkMsgProc: TMarkMsgProc;
  StateShow: TStateShowProc): Boolean;
label fine;
var
  MsgNo: Integer;
  PrepMsg: String;
  PagerProps: TPagerProps;
  R: Word;
  block: Integer;
  MsgList,
  BlockList,
  KindList: TList;
begin
  Result:= False;
  MsgList:= TList.Create;
  BlockList:= TList.Create;
  KindList:= TList.Create;
  { --- carrier detect --- }
  StateShow(' ');
  if (cmThruModem in ATerminalPars.comPars.PgConOptSet) and
    (ATerminalPars.comPars.Phone > '') then begin
    StateShow(' ,   30 ..');
    if AP_CallModem(AComm, ATerminalPars.comPars.Phone, 30) then begin
      StateShow('   ');
    end else begin
      StateShow('   ');
    end;
  end;
  if not AP_WaitCD(AComm, ATerminalPars.comPars.CDWaitSec) then begin
    StateShow('   .');
    goto fine;
  end;
  StateShow(' ,  ');

  {---   ID= --- }
  if not AP_WaitPrompt(AComm, ATerminalPars.comPars, #13,
    WEBStyleAndControlString2ASCII(ATerminalPars.comPars.LoginPrompt), 9, 3) then begin
    AComm.DTR:= True;
    AComm.RTS:= True;
    StateShow(' .');
    goto fine;
  end;
  StateShow(' , (/<CR>),  .');

  { ---  PG1 --- }
  if not AP_TryLogon(AComm, ATerminalPars.comPars, 2, StateShow) then begin
    StateShow('  , - .');
    goto fine;
  end;
  StateShow(' ,     .');

  { ---    1  --- }
  if not AP_WaitReadyTransaction(AComm, ATerminalPars.comPars, WEBStyleAndControlString2ASCII(
    ATerminalPars.comPars.ReadyTransactionStr), 10) then begin
    StateShow('      1- .');
    goto fine;
  end;
  StateShow('    ,  .');

  { ---     ,   --- }
  //    MsgNo    Msg.
  //  4     
  while AGetMsgProc(4, MsgNo, PrepMsg, PagerProps) do begin
    TransactionFillMessages(PrepMsg,
    ATerminalPars,
    PagerProps.MsgLen, MsgList);
    TransactionFillBlocks(PrepMsg, MsgList, ATerminalPars.tapBlockLen, BlockList, KindList);
    { ..      }
    if not (cmNoSendMsg in ATerminalPars.comPars.PgConOptSet) then begin
      R:= 0; //       -    .    
      {    }
      for block:= 1 to BlockList.Count do begin
        {  }
        R:= AP_SendTAPBlock(AComm,
          GenerateMessageBlock(MsgNo, block-1, PrepMsg,
          ATerminalPars, MsgList, BlockList, KindList));
        if R <> 0
        then Break;
      end;
      if R <> 0 then begin
        //     
        AMarkMsgProc(MsgNo, False);
        StateShow(ErrorSendBlockString(R))
      end else begin
        // ,    
        AMarkMsgProc(MsgNo, True);
        StateShow(' ')
      end;
    end;
  end;
  { ---   --- }
  StateShow(' ..');
  if AP_DisconnectTransaction(AComm, 10) then begin
    StateShow('  ');
  end else begin
    StateShow('     (  ,..)');
  end;
fine:
  // AMSComm.Output:= #13;
  MsgList.Free;
  BlockList.Free;
  KindList.Free;
  Result:= True;
end;

  //      TTAPsPars    TAPDComPort
  //    
constructor TTAPTerminals.Create;
begin
  FAPDComPorts:= Nil;
  inherited Create;
end;

//      Count
procedure TTAPTerminals.SetCount(ACount: Word);
var
  i, OldCount: Word;
begin
  OldCount:= Count;
  try
    ReAllocMem(FAPDComPorts, ACount * SizeOf(TAPDComPort));
    inherited SetCount(ACount);
  except
  end;
  if ACount > OldCount then begin
    for i:= OldCount to ACount-1 do begin
      FAPDComPorts^[i]:= TAPDComPort.Create(Nil);
    end;
  end else begin
    if ACount > 0 then begin
      for i:= OldCount-1 downto ACount do begin
        FAPDComPorts^[i].Free;
      end;
    end;
  end;
end;

function  TTAPTerminals.GetComPort(AIndex: Word): TAPDComPort;
begin
  if (AIndex >= Count) or (FAPDComPorts = Nil) then begin
    raise Exception.Create(' : '+IntToStr(AIndex));
    Exit;
  end;
  Result:= FAPDComPorts^[AIndex];
end;

//       
procedure TTAPTerminals.SetTerminalPar(AIndex: Word; AValue: TTerminalPar);
begin
  inherited SetTerminalPar(AIndex, AValue);
  AP_SetComPort(FAPDComPorts^[AIndex], Params[AIndex].comPars);
end;

//      (write  com- )
// -     
procedure TTAPTerminals.SetComPort(AIndex: Word; AComPort: TAPDComPort);
begin
  if (AIndex >= Count) or (FAPDComPorts = Nil) then begin
    raise Exception.Create(' : '+IntToStr(AIndex));
    Exit;
  end;
  FAPDComPorts^[AIndex].Assign(AComPort);
end;

end.
