unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, ComCtrls, OleCtrls, Buttons, ToolWin, Isp3,
  printers, DdeMan;

const
  CM_HOMEPAGEREQUEST = WM_USER + $1000;
  DDEAPPLICATION = 'Eoper.exe';
  DDESERVICE     = 'EOPER';
  DDETOPIC       = 'convOperator';
  DDEITEM        = 'itemAbonent';

type
  TFormBrowser = class(TForm)
    StatusBar1: TStatusBar;
    MainMenu1: TMainMenu;
    MFile: TMenuItem;
    MFileExit: TMenuItem;
    View1: TMenuItem;
    MViewDocSource: TMenuItem;
    NavigatorImages: TImageList;
    NavigatorHotImages: TImageList;
    LinksImages: TImageList;
    LinksHotImages: TImageList;
    CoolBar1: TCoolBar;
    ToolBar1: TToolBar;
    TBBack: TToolButton;
    TBForward: TToolButton;
    TBStop: TToolButton;
    TBRefresh: TToolButton;
    ToolBar2: TToolBar;
    ToolButton6: TToolButton;
    Animate1: TAnimate;
    URLs: TComboBox;
    HTML1: THTML;
    mN1: TMenuItem;
    MViewToolbar: TMenuItem;
    MViewStatusbar: TMenuItem;
    mN2: TMenuItem;
    MViewRefresh: TMenuItem;
    MViewStop: TMenuItem;
    TBDDESend: TToolButton;
    FFileDDE: TMenuItem;
    mN3: TMenuItem;
    DdeClientConv1: TDdeClientConv;
    DdeClientItem1: TDdeClientItem;
    TBPrint: TToolButton;
    procedure MFileExitClick(Sender: TObject);
    procedure MViewDocSourceClick(Sender: TObject);
    procedure TBStopClick(Sender: TObject);
    procedure HTML1BeginRetrieval(Sender: TObject);
    procedure HTML1EndRetrieval(Sender: TObject);
    procedure URLsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure LinksClick(Sender: TObject);
    procedure TBRefreshClick(Sender: TObject);
    procedure TBBackClick(Sender: TObject);
    procedure TBForwardClick(Sender: TObject);
    procedure HTML1DoRequestDoc(Sender: TObject; const URL: WideString;
      const Element: HTMLElement; const DocInput: DocInput;
      var EnableDefault: WordBool);
    procedure FormDestroy(Sender: TObject);
    procedure URLsClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MViewToolbarClick(Sender: TObject);
    procedure MViewStatusbarClick(Sender: TObject);
    procedure DdeClientItem1Change(Sender: TObject);
    procedure TBDDESendClick(Sender: TObject);
    procedure FFileDDEClick(Sender: TObject);
    procedure TBPrintClick(Sender: TObject);
  private
    HistoryIndex: Integer;
    HistoryList: TStringList;
    UpdateCombo: Boolean;
    LinkEstablished: Boolean;
    procedure FindAddress;
    procedure HomePageRequest(var message: tmessage); message CM_HOMEPAGEREQUEST;
    procedure ViewDDEStatus;
    procedure Set_DDE_Link(ATurnOn: Boolean);
  end;

var
  FormBrowser: TFormBrowser;

implementation

uses
  DocSrc;

{$R *.DFM}

procedure TFormBrowser.MFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormBrowser.FindAddress;
begin
  HTML1.RequestDoc(URLs.Text);
end;

procedure TFormBrowser.MViewDocSourceClick(Sender: TObject);
begin
  with DocSourceFrm do begin
    Show;
    Memo1.Lines.Clear;
    Memo1.Lines.Add(AdjustLineBreaks(HTML1.SourceText));
    Memo1.SelStart := 0;
    SendMessage(Memo1.Handle, EM_ScrollCaret, 0, 0);
  end;
end;

procedure TFormBrowser.TBStopClick(Sender: TObject);
begin
  HTML1.Cancel('Cancel');
  HTML1EndRetrieval(nil);
end;

procedure TFormBrowser.HTML1BeginRetrieval(Sender: TObject);
begin
  { Turn the stop button dark red }
  TBStop.ImageIndex := 4;
  { Play the avi from the first frame indefinitely }
  Animate1.Active := True;
end;

procedure TFormBrowser.HTML1EndRetrieval(Sender: TObject);
begin
  { Turn the stop button grey }
  TBStop.ImageIndex := 2;
  { Stop the avi and show the first frame }
  Animate1.Active := False;
end;

procedure TFormBrowser.URLsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_Return then
  begin
    UpdateCombo := True;
    FindAddress;
  end;
end;

procedure TFormBrowser.URLsClick(Sender: TObject);
begin
  UpdateCombo := True;
  FindAddress;
end;

procedure TFormBrowser.LinksClick(Sender: TObject);
begin
  if (Sender as TToolButton).Hint = '' then Exit;
  URLs.Text := (Sender as TToolButton).Hint;
  FindAddress;
end;

procedure TFormBrowser.TBRefreshClick(Sender: TObject);
begin
  FindAddress;
end;

procedure TFormBrowser.TBBackClick(Sender: TObject);
begin
  URLs.Text := HistoryList[HistoryIndex - 1];
  FindAddress;
end;

procedure TFormBrowser.TBForwardClick(Sender: TObject);
begin
  URLs.Text := HistoryList[HistoryIndex + 1];
  FindAddress;
end;

procedure TFormBrowser.HTML1DoRequestDoc(Sender: TObject;
  const URL: WideString; const Element: HTMLElement;
  const DocInput: DocInput; var EnableDefault: WordBool);
var
  NewIndex: Integer;
begin
  NewIndex := HistoryList.IndexOf(URL);
  if NewIndex = -1 then
  begin
    { Remove entries in HistoryList between last address and current address }
    if (HistoryIndex >= 0) and (HistoryIndex < HistoryList.Count - 1) then
      while HistoryList.Count > HistoryIndex do
        HistoryList.Delete(HistoryIndex);
    HistoryIndex := HistoryList.Add(URL);
  end
  else
    HistoryIndex := NewIndex;
  if HistoryList.Count > 0 then
  begin
    TBForward.Enabled := HistoryIndex < HistoryList.Count - 1;
    TBBack.Enabled := HistoryIndex > 0;
  end
  else
  begin
    TBForward.Enabled := False;
    TBBack.Enabled := False;
  end;
  if UpdateCombo then
  begin
    UpdateCombo := False;
    NewIndex := URLs.Items.IndexOf(URL);
    if NewIndex = -1 then
      URLs.Items.Insert(0, URL)
    else
      URLs.Items.Move(NewIndex, 0);
  end;
  URLs.Text := URL;
  Statusbar1.Panels[1].Text := URL;
end;

procedure TFormBrowser.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Shift = [ssAlt] then
    if (Key = VK_RIGHT) and TBForward.Enabled then
      TBForward.Click
    else if (Key = VK_LEFT) and TBBack.Enabled then
      TBBack.Click;
end;

procedure TFormBrowser.MViewToolbarClick(Sender: TObject);
begin
  with Sender as TMenuItem do
  begin
    Checked := not Checked;
    Coolbar1.Visible := Checked;
  end;
end;

procedure TFormBrowser.MViewStatusbarClick(Sender: TObject);
begin
  with Sender as TMenuItem do
  begin
    Checked := not Checked;
    StatusBar1.Visible := Checked;
  end;
end;

procedure TFormBrowser.HomePageRequest(var Message: TMessage);
begin
  URLs.Text := 'http://localhost';
  UpdateCombo := True;
  FindAddress;
end;

procedure TFormBrowser.FormCreate(Sender: TObject);
begin
  HistoryIndex := -1;
  HistoryList := TStringList.Create;
  { Load the animation from the AVI file in the startup directory.  An
    alternative to this would be to create a .RES file including the cool.avi
    as an AVI resource and use the ResName or ResId properties of Animate1 to
    point to it. }
  Animate1.FileName:= ExtractFilePath(Application.ExeName) + 'cool.avi';
  { Find the home page - needs to be posted because HTML control hasn't been
    registered yet. }
  PostMessage(Handle, CM_HOMEPAGEREQUEST, 0, 0);
  with HTML1 do begin
    Font.Charset:= RUSSIAN_CHARSET;
    FixedFont.Charset:= RUSSIAN_CHARSET;
    Heading1Font.Charset:= RUSSIAN_CHARSET;
    Heading2Font.Charset:= RUSSIAN_CHARSET;
    Heading3Font.Charset:= RUSSIAN_CHARSET;
    Heading4Font.Charset:= RUSSIAN_CHARSET;
    Heading5Font.Charset:= RUSSIAN_CHARSET;
    Heading6Font.Charset:= RUSSIAN_CHARSET;
  end;
  LinkEstablished:= False;
end;

procedure TFormBrowser.FormDestroy(Sender: TObject);
begin
  HistoryList.Free;
  Set_DDE_Link(False);
end;

procedure TFormBrowser.ViewDDEStatus;
begin
  FFileDDE.Checked:= LinkEstablished;
  if LinkEstablished
  then TBDDESend.ImageIndex:= 5
  else TBDDESend.ImageIndex:= 6;
end;

procedure TFormBrowser.Set_DDE_Link(ATurnOn: Boolean);
begin
  // nothing to do, exit
  if LinkEstablished = ATurnOn
  then Exit;
  if ATurnOn then begin
    DdeClientItem1.DdeItem:= DDEITEM;
    // DdeClientItem1.DdeConv:= DdeClientConv1;
    LinkEstablished:= DdeClientConv1.SetLink(DDESERVICE, DDETOPIC) and
      DdeClientConv1.OpenLink;
    DdeClientItem1.DdeItem:= DDEITEM;
  end else begin
    DdeClientConv1.CloseLink;
    LinkEstablished:= False;
  end;
end;

procedure TFormBrowser.DdeClientItem1Change(Sender: TObject);
var
  S: String;
begin
  {}
  S:= DdeClientItem1.Text;
  if S <> ''
  then Statusbar1.Panels[0].Text:= S;
end;

procedure TFormBrowser.TBDDESendClick(Sender: TObject);
var
  S: String;
  p: array[0..4095] of Char;
begin
  {}
  Set_DDE_Link(True);
  ViewDDEStatus;
  if LinkEstablished then begin
    S:= HTML1.GetPlainText(HTML1.HasSelection, false);
    StrPCopy(P, Copy(S, 1, 4095));
    DdeClientConv1.ExecuteMacro(P, False);
    // StatusBar1.Panels[0].Text:= '   ""';
  end;
end;

procedure TFormBrowser.FFileDDEClick(Sender: TObject);
begin
  {}
  Set_DDE_Link(not LinkEstablished);
  ViewDDEStatus;
end;

procedure TFormBrowser.TBPrintClick(Sender: TObject);
begin
  {  }
  Printer.BeginDoc;
  html1.AutoPrint(Printers.Printer.Canvas.Handle);
  Printer.EndDoc;
end;

end.
